// ----------------------------------------------------------------------------
// This reference design and source code is being provided on an "as-is" basis 
// and as an accommodation, and therefore all warranties, representations or 
// guarantees of any kind (whether express, implied or statutory) including, 
// without limitation, warranties of merchantability, non-infringement, or  
// fitness for a particular purpose, are specifically disclaimed.
//
// This source code may only be used in an Altera programmable logic device
// and may not be distributed without permission from Macnica Americas, Inc.  
// It is provided free of royalties or fees of any kind.
// ----------------------------------------------------------------------------
//
// Description  | Demonstrate ADC reading analog inputs including microphone
//              | 
//              | I2C Register Map:
//              | Base = 0x30 (Wr base: 0x60, Rd base: 0x61)
//              |		00: AutoReg
//              |			The auto reg is driven by HW and polled by software for indicator control
//              |			indicator1 = button 1 active, autoreg --> 0x55
//              | 		indicator2 = button 2 active, autoreg --> 0xaa
//              | 	01: ADC start/stop
//              | 		[7:1]	unused
//              | 		[0]   1 = start, 0 = stop
//              | 	02: Channel 0 ADC voltage (upper)
//              | 		12-bit analog data, in mV
//              | 		[7:4] unused
//              | 		[3:0] Channel 0 ADC voltage (bits 9-12)
//              | 	03: Channel 0 ADC voltage (lower)
//              | 		12-bit analog data, in mV
//              | 		[7:0] Channel 0 ADC voltage (bits 1-8)
//              | 	04: Channel 1 ADC voltage (upper)
//              | 		12-bit analog data, in mV
//              | 		[7:4] unused
//              | 		[3:0] Channel 1 ADC voltage (bits 9-12)
//              | 	05: Channel 1 ADC voltage (lower)
//              | 		12-bit analog data, in mV
//              | 		[7:0] Ch 1 ADC voltage (bits 1-8)
//              | 	06: Channel 2 ADC voltage (upper)
//              | 		12-bit analog data, in mV
//              | 		[7:4] unused
//              | 		[3:0] Channel 2 ADC voltage (bits 9-12)
//              | 	07: Channel 2 ADC voltage (lower)
//              | 		12-bit analog data, in mV
//              | 		[7:0] Ch 2 ADC voltage (bits 1-8)
//              | 	08: Channel 3 ADC voltage (upper)
//              | 		12-bit analog data, in mV
//              | 		[7:4] unused
//              | 		[3:0] Channel 3 ADC voltage (bits 9-12)
//              | 	09: Channel 3 ADC voltage (lower)
//              | 		12-bit analog data, in mV
//              | 		[7:0] Channel 3 ADC voltage (bits 1-8)
//              | 	0a: Channel 4 ADC voltage (upper)
//              | 		12-bit analog data, in mV
//              | 		[7:4] unused
//              | 		[3:0] Channel 4 ADC voltage (bits 9-12)
//              | 	0b: Channel 4 ADC voltage (lower)
//              | 		12-bit analog data, in mV
//              | 		[7:0] Channel 4 ADC voltage (bits 1-8)
//              | 	0c: Channel 5 ADC voltage (upper)
//              | 		12-bit analog data, in mV
//              | 		[7:4] unused
//              | 		[3:0] Channel 5 ADC voltage (bits 9-12)
//              | 	0d: Channel 5 ADC voltage (lower)
//              | 		12-bit analog data, in mV
//              | 		[7:0] Channel 5 ADC voltage (bits 1-8)
//              | 	0e: Microphone ADC voltage (upper)
//              | 		12-bit analog data, in mV
//              | 		[7:4] unused
//              | 		[3:0] Microphone ADC voltage (bits 9-12)
//              | 	0f: Microphone ADC voltage (lower)
//              | 		12-bit analog data, in mV
//              | 		[7:0] Microphone ADC voltage (bits 1-8)
//              | 	10: Microphone max ADC voltage (upper)
//              | 		12-bit analog data, in mV
//              | 		[7:4] unused
//              | 		[3:0] Microphone max ADC voltage (bits 9-12)
//              | 	11: Microphone max ADC voltage (lower)
//              | 		12-bit analog data, in mV
//              | 		[7:0] Microphone max ADC voltage (bits 1-8)
//              | 
// Formatting   | Tabs set at 3 spaces.  Port names 'UPPER' case.
//              | Internal wires and registers are 'lower' case.
//              | Module names 'lower' case to match lower case filename.v
//              | Parameters are first character 'Upper' case.
//              | Active low signals are identified with '_n' or '_N'
//              | appended to the wire, register, or port name.
//              | 
//              | _r suffix on a register name indicates a simple
//              | re-registering of another wire or register.
//              | 
//              | _rr suffix on a register name indicates a second
//              | re-registering of a register.
//              | 
//              | _metastab indicates clock domain crossing
//
// ----------------------------------------------------------------------------

`timescale 1ns/1ps

//--------------------------
//	Declaration and ports
//-----------
module top (
	
	//global
	input					IN_CLOCK,
	
	//i2c
	input					IN_SCL,
	inout					INOUT_SDA,
	
	//gpio
	output	[7:0]		OUT_LED,
	input		[2:0]		IN_SWITCH,
	input		[1:0]		IN_BUTTON_N
);


//--------------------------
//	Regs and Wires
//-----------
	reg					reset_n_r;
	reg					reset_n_rr;
	wire					clk_core;
	wire					clk_adc;
	wire					locked;
	reg			      adc_response_valid;
	reg [4:0]	      adc_response_channel;
	reg [11:0]	      adc_response_data;
	reg		         adc_response_startofpacket;
	reg			      adc_response_endofpacket;
	reg		         adc_sequencer_csr_address;
	reg			      adc_sequencer_csr_read;
	reg			      adc_sequencer_csr_write;
	reg  [31:0]	      adc_sequencer_csr_writedata;
	reg  [31:0]	      adc_sequencer_csr_readdata;
	wire [7:0]        adc_cmd;
	reg  [7:0]        autoreg;	
	reg  [7:0]	      reg_ch0_upper, reg_ch0_lower;
	reg  [7:0]	      reg_ch1_upper, reg_ch1_lower;
	reg  [7:0]	      reg_ch2_upper, reg_ch2_lower;
   reg  [7:0]	      reg_ch3_upper, reg_ch3_lower;
   reg  [7:0]	      reg_ch4_upper, reg_ch4_lower;
   reg  [7:0]	      reg_ch5_upper, reg_ch5_lower;
   reg  [7:0]	      reg_microphone_upper, reg_microphone_lower;
   reg  [7:0]	      reg_microphone_max_upper, reg_microphone_max_lower;
   reg  [13:0]       mic_max_cnt;
   reg  [15:0]       reg_mic_max_tmp;
   reg               adc_run, adc_run_r, adc_run_rr, adc_stop;
	reg					initial_turnon, initial_turnon_r, initial_turnon_rr;
   wire [11:0]       microphone_max;
   reg  [11:0]       adc_ch0_raw_data;
   reg  [11:0]       adc_ch1_raw_data;
   reg  [11:0]       adc_ch2_raw_data;
   reg  [11:0]       adc_ch3_raw_data;
   reg  [11:0]       adc_ch4_raw_data;
   reg  [11:0]       adc_ch5_raw_data;
   wire [23:0]       adc_ch0_volt_data;
   wire [23:0]       adc_ch1_volt_data;
   wire [23:0]       adc_ch2_volt_data;
   wire [23:0]       adc_ch3_volt_data;
   wire [23:0]       adc_ch4_volt_data;
   wire [23:0]       adc_ch5_volt_data;


//--------------------------
//	PLL
//-----------
pll pll_inst (
   .inclk0 ( IN_CLOCK ),
   .c0 ( clk_adc ),	// 10 MHz clock dedicated to ADC
   .c1 ( clk_core ),	// 50 MHz sytem clock
   .locked ( locked )
);


//--------------------------
//	Reset - Synchronous deassert after PLL locked
//-----------
//Synchronize Reset
always @(posedge IN_CLOCK)
	begin
	if (!locked)
		begin
		reset_n_r <= 1'b0;
		reset_n_rr <= 1'b0;
		end
	else
		begin
		reset_n_r <= 1'b1;
		reset_n_rr <= reset_n_r;
		end
	end


//--------------------------
//	Encode the auto register
// The auto reg is driven by HW and polled by software for indicator control
//	indicator1 = button 1 active, autoreg --> 0x55
//	indicator2 = button 2 active, autoreg --> 0xaa
//-----------
always @(posedge clk_core)
	begin
	if (!reset_n_rr)
		begin
		autoreg[7:0] <= 8'h00;
		end
	else
		begin
		case (~IN_BUTTON_N[1:0])
			2'b01		: autoreg <= 8'h55;
			2'b10		: autoreg <= 8'haa;
			default	: autoreg <= 8'h00;
		endcase
		end
	end


//--------------------------
//	The I2C Slave
//-----------
i2cSlave i2c_inst(
	.clk (clk_core),
	.rst (!reset_n_rr),
	.sda (INOUT_SDA),
	.scl (IN_SCL),
	.reg0_auto_in (autoreg[7:0]),
   .reg1_adc_cmd_out (adc_cmd), 
   .reg2_ch0_upper_in (reg_ch0_upper),
   .reg3_ch0_lower_in (reg_ch0_lower),
   .reg4_ch1_upper_in (reg_ch1_upper),
   .reg5_ch1_lower_in (reg_ch1_lower),
   .reg6_ch2_upper_in (reg_ch2_upper),
   .reg7_ch2_lower_in (reg_ch2_lower),
   .reg8_ch3_upper_in (reg_ch3_upper),
   .reg9_ch3_lower_in (reg_ch3_lower),
   .reg10_ch4_upper_in (reg_ch4_upper),
   .reg11_ch4_lower_in (reg_ch4_lower),
   .reg12_ch5_upper_in (reg_ch5_upper),
   .reg13_ch5_lower_in (reg_ch5_lower),	
   .reg14_microphone_upper_in (reg_microphone_upper),
   .reg15_microphone_lower_in (reg_microphone_lower),
   .reg16_microphone_max_upper_in (reg_microphone_max_upper),
   .reg17_microphone_max_lower_in (reg_microphone_max_lower)
);


//--------------------------
//	ADC Commands
//-----------

// Detect start and stop ADC commands from I2C
always @(posedge clk_core or negedge reset_n_rr) begin
   if (!reset_n_rr) begin
      adc_run_r <= 1'b0;
      adc_run_rr <= 1'b0;
      adc_run <= 1'b0;
		initial_turnon <= 1'b1;
		initial_turnon_r <= 1'b0;
		initial_turnon_rr <= 1'b0;
   end else begin
		initial_turnon <= 1'b0;
		initial_turnon_r <= initial_turnon;
		initial_turnon_rr <= initial_turnon_r;
      adc_run_r <= adc_cmd[0];
      adc_run_rr <= adc_run_r;
      if (initial_turnon_rr) begin
			adc_run <= 1'b1;
		end else if (adc_run_r & ~adc_run_rr) begin  // Rising edge detect
         adc_run <= 1'b1;
      end else begin
         adc_run <= 1'b0;
      end 
		if (!adc_run_r & adc_run_rr) begin  // Falling edge detect
         adc_stop <= 1'b1;
      end else begin
         adc_stop <= 1'b0;
      end 
   end
end

// Issue ADC sequencer start and stop commands
always @(posedge clk_core or negedge reset_n_rr) begin
   if (!reset_n_rr) begin
      adc_sequencer_csr_address <= 1'b0;
      adc_sequencer_csr_read <= 1'b0;
      adc_sequencer_csr_write <= 1'b0;
      adc_sequencer_csr_writedata <= 32'b0;
   end else begin
      if (adc_run) begin
  	      adc_sequencer_csr_address <= 1'b0;
	      adc_sequencer_csr_read <= 1'b0;
	      adc_sequencer_csr_write <= 1'b1;
	      adc_sequencer_csr_writedata <= 32'b1;
      end else if (adc_stop) begin
  	      adc_sequencer_csr_address <= 1'b0;
	      adc_sequencer_csr_read <= 1'b0;
	      adc_sequencer_csr_write <= 1'b1;
	      adc_sequencer_csr_writedata <= 32'b0;
	   end else begin
	 	   adc_sequencer_csr_address <= 1'b0;
	      adc_sequencer_csr_read <= 1'b0;
	      adc_sequencer_csr_write <= 1'b0;
	      adc_sequencer_csr_writedata <= 32'b0;
      end
   end
end


//--------------------------
//	ADC- 7 channels: ch 0-5, microphone
//-----------

adc u0 (
	.clk_clk                     (clk_core),                    //               clk.clk
	.reset_reset_n               (reset_n_rr),                  //             reset.reset_n
	.adc_pll_clock_clk           (clk_adc),                    	//     adc_pll_clock.clk
	.adc_pll_locked_export       (locked),                   	//    adc_pll_locked.export
	.adc_response_valid          (adc_response_valid),     		//      adc_response.valid
	.adc_response_channel        (adc_response_channel),        //                  .channel
	.adc_response_data           (adc_response_data),           //                  .data
	.adc_response_startofpacket  (adc_response_startofpacket),  //                  .startofpacket
	.adc_response_endofpacket    (adc_response_endofpacket),    //                  .endofpacket
	.adc_sequencer_csr_address   (adc_sequencer_csr_address),   // adc_sequencer_csr.address
	.adc_sequencer_csr_read      (adc_sequencer_csr_read),      //                  .read
	.adc_sequencer_csr_write     (adc_sequencer_csr_write),     //                  .write
	.adc_sequencer_csr_writedata (adc_sequencer_csr_writedata), //                  .writedata
	.adc_sequencer_csr_readdata  (adc_sequencer_csr_readdata)   //                  .readdata
);


//--------------------------
//	Capture ADC channel data
//-----------

always @(posedge clk_core or negedge reset_n_rr) begin
   if (!reset_n_rr) begin
		adc_ch0_raw_data <= 12'b0;
		adc_ch1_raw_data <= 12'b0;
		adc_ch2_raw_data <= 12'b0;
		adc_ch3_raw_data <= 12'b0;
		adc_ch4_raw_data <= 12'b0;
		adc_ch5_raw_data <= 12'b0;
      reg_microphone_upper <= 8'b0;
      reg_microphone_lower <= 8'b0;
   end else begin
      if (adc_response_valid) begin 
         case (adc_response_channel)
            5'h00: adc_ch0_raw_data <= adc_response_data;
            5'h01: adc_ch1_raw_data <= adc_response_data;
            5'h02: adc_ch2_raw_data <= adc_response_data;
            5'h03: adc_ch3_raw_data <= adc_response_data;
            5'h04: adc_ch4_raw_data <= adc_response_data;
            5'h05: adc_ch5_raw_data <= adc_response_data;
            5'h06: {reg_microphone_upper, reg_microphone_lower} <= adc_response_data;				
         endcase
      end
   end
end


//--------------------------
//	Multiplication and divison for ADC ch 0-5
//	Converts from raw bin data to equivalent mV 
// result = data * 825 / 1024 (for 3.3V internal reference voltage)
//-----------

mult	mult_ch0_inst (
	.dataa ( adc_ch0_raw_data ),
	.result ( adc_ch0_volt_data )
);

mult	mult_ch1_inst (
	.dataa ( adc_ch1_raw_data ),
	.result ( adc_ch1_volt_data )
);

mult	mult_ch2_inst (
	.dataa ( adc_ch2_raw_data ),
	.result ( adc_ch2_volt_data )
);

mult	mult_ch3_inst (
	.dataa ( adc_ch3_raw_data ),
	.result ( adc_ch3_volt_data )
);

mult	mult_ch4_inst (
	.dataa ( adc_ch4_raw_data ),
	.result ( adc_ch4_volt_data )
);

mult	mult_ch5_inst (
	.dataa ( adc_ch5_raw_data ),
	.result ( adc_ch5_volt_data )
);

always @(posedge clk_core or negedge reset_n_rr) begin
	if (!reset_n_rr) begin
      reg_ch0_upper <= 8'b0;
      reg_ch0_lower <= 8'b0;	
      reg_ch1_upper <= 8'b0;
      reg_ch1_lower <= 8'b0;	
      reg_ch2_upper <= 8'b0;
      reg_ch2_lower <= 8'b0;	
      reg_ch3_upper <= 8'b0;
      reg_ch3_lower <= 8'b0;	
      reg_ch4_upper <= 8'b0;
      reg_ch4_lower <= 8'b0;	
      reg_ch5_upper <= 8'b0;
      reg_ch5_lower <= 8'b0;	
	end else begin
	   if (adc_response_endofpacket) begin
			{reg_ch0_upper, reg_ch0_lower} <= {4'b0, adc_ch0_volt_data[21:10]}; // div by 1024
			{reg_ch1_upper, reg_ch1_lower} <= {4'b0, adc_ch1_volt_data[21:10]}; // div by 1024
			{reg_ch2_upper, reg_ch2_lower} <= {4'b0, adc_ch2_volt_data[21:10]}; // div by 1024
			{reg_ch3_upper, reg_ch3_lower} <= {4'b0, adc_ch3_volt_data[21:10]}; // div by 1024
			{reg_ch4_upper, reg_ch4_lower} <= {4'b0, adc_ch4_volt_data[21:10]}; // div by 1024
			{reg_ch5_upper, reg_ch5_lower} <= {4'b0, adc_ch5_volt_data[21:10]}; // div by 1024
		end
	end
end


//--------------------------
// Captures max microphone value and hold for ~0.1s
//-----------

always @(posedge clk_core or negedge reset_n_rr) begin
	if (!reset_n_rr) begin
		reg_microphone_max_upper <= 8'b0;
		reg_microphone_max_lower <= 8'b0;
		reg_mic_max_tmp <= 16'b0;
	   mic_max_cnt <= 14'b0;					
	end else begin
	   if (adc_response_endofpacket) begin
		   if (mic_max_cnt == {14{1'b1}}) begin
			   mic_max_cnt <= 14'b0;				
				reg_microphone_max_upper <= reg_mic_max_tmp[15:8];
				reg_microphone_max_lower <= reg_mic_max_tmp[7:0]; 
				reg_mic_max_tmp <= 14'b0;
			end else begin
				mic_max_cnt <= mic_max_cnt + 14'b1;
			end
			if ({reg_microphone_upper, reg_microphone_lower} > reg_mic_max_tmp) begin
				reg_mic_max_tmp <= {reg_microphone_upper, reg_microphone_lower};
			end
		end
	end
end


//--------------------------
// Output microphone max level to LEDs
//-----------

assign microphone_max = {reg_microphone_max_upper[3:0], reg_microphone_max_lower};

assign OUT_LED[7:0] = ~((microphone_max > 12'hA0D) ? 8'hff :
								(microphone_max > 12'h858) ? 8'h7f :
								(microphone_max > 12'h77E) ? 8'h3f :
								(microphone_max > 12'h711) ? 8'h1f :
								(microphone_max > 12'h6DA) ? 8'h0f :
								(microphone_max > 12'h6BF) ? 8'h07 :
								(microphone_max > 12'h6B1) ? 8'h03 : 
								8'h01);
								
								
								
								

endmodule
